"Main interface for textract service Client"
from __future__ import annotations

from typing import Any, Dict, List
from botocore.client import BaseClient
from botocore.exceptions import ClientError as Boto3ClientError
from mypy_boto3.type_defs import Literal

# pylint: disable=import-self
import mypy_boto3_textract.client as client_scope
from mypy_boto3_textract.type_defs import (
    ClientAnalyzeDocumentDocumentTypeDef,
    ClientAnalyzeDocumentHumanLoopConfigTypeDef,
    ClientAnalyzeDocumentResponseTypeDef,
    ClientDetectDocumentTextDocumentTypeDef,
    ClientDetectDocumentTextResponseTypeDef,
    ClientGetDocumentAnalysisResponseTypeDef,
    ClientGetDocumentTextDetectionResponseTypeDef,
    ClientStartDocumentAnalysisDocumentLocationTypeDef,
    ClientStartDocumentAnalysisNotificationChannelTypeDef,
    ClientStartDocumentAnalysisResponseTypeDef,
    ClientStartDocumentTextDetectionDocumentLocationTypeDef,
    ClientStartDocumentTextDetectionNotificationChannelTypeDef,
    ClientStartDocumentTextDetectionResponseTypeDef,
)


__all__ = ("Client",)


class Client(BaseClient):
    exceptions: client_scope.Exceptions

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def analyze_document(
        self,
        Document: ClientAnalyzeDocumentDocumentTypeDef,
        FeatureTypes: List[Literal["TABLES", "FORMS"]],
        HumanLoopConfig: ClientAnalyzeDocumentHumanLoopConfigTypeDef = None,
    ) -> ClientAnalyzeDocumentResponseTypeDef:
        """
        Analyzes an input document for relationships between detected items.

        The types of information returned are as follows:

        * Form data (key-value pairs). The related information is returned in two  Block objects,
        each of type ``KEY_VALUE_SET`` : a KEY ``Block`` object and a VALUE ``Block`` object. For
        example, *Name: Ana Silva Carolina* contains a key and value. *Name:* is the key. *Ana Silva
        Carolina* is the value.

        * Table and table cell data. A TABLE ``Block`` object contains information about a detected
        table. A CELL ``Block`` object is returned for each cell in a table.

        * Lines and words of text. A LINE ``Block`` object contains one or more WORD ``Block``
        objects. All lines and words that are detected in the document are returned (including text
        that doesn't have a relationship with the value of ``FeatureTypes`` ).

        Selection elements such as check boxes and option buttons (radio buttons) can be detected in
        form data and in tables. A SELECTION_ELEMENT ``Block`` object contains information about a
        selection element, including the selection status.

        You can choose which type of analysis to perform by specifying the ``FeatureTypes`` list.

        The output is returned in a list of ``Block`` objects.

         ``AnalyzeDocument`` is a synchronous operation. To analyze documents asynchronously, use
         StartDocumentAnalysis .

        For more information, see `Document Text Analysis
        <https://docs.aws.amazon.com/textract/latest/dg/how-it-works-analyzing.html>`__ .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/textract-2018-06-27/AnalyzeDocument>`_

        **Request Syntax**
        ::

          response = client.analyze_document(
              Document={
                  'Bytes': b'bytes',
                  'S3Object': {
                      'Bucket': 'string',
                      'Name': 'string',
                      'Version': 'string'
                  }
              },
              FeatureTypes=[
                  'TABLES'|'FORMS',
              ],
              HumanLoopConfig={
                  'HumanLoopName': 'string',
                  'FlowDefinitionArn': 'string',
                  'DataAttributes': {
                      'ContentClassifiers': [
                          'FreeOfPersonallyIdentifiableInformation'|'FreeOfAdultContent',
                      ]
                  }
              }
          )
        :type Document: dict
        :param Document: **[REQUIRED]**

          The input document as base64-encoded bytes or an Amazon S3 object. If you use the AWS CLI
          to call Amazon Textract operations, you can't pass image bytes. The document must be an
          image in JPEG or PNG format.

          If you're using an AWS SDK to call Amazon Textract, you might not need to base64-encode
          image bytes that are passed using the ``Bytes`` field.

          - **Bytes** *(bytes) --*

            A blob of base64-encoded document bytes. The maximum size of a document that's provided
            in a blob of bytes is 5 MB. The document bytes must be in PNG or JPEG format.

            If you're using an AWS SDK to call Amazon Textract, you might not need to base64-encode
            image bytes passed using the ``Bytes`` field.

          - **S3Object** *(dict) --*

            Identifies an S3 object as the document source. The maximum size of a document that's
            stored in an S3 bucket is 5 MB.

            - **Bucket** *(string) --*

              The name of the S3 bucket.

            - **Name** *(string) --*

              The file name of the input document. Synchronous operations can use image files that
              are in JPEG or PNG format. Asynchronous operations also support PDF format files.

            - **Version** *(string) --*

              If the bucket has versioning enabled, you can specify the object version.

        :type FeatureTypes: list
        :param FeatureTypes: **[REQUIRED]**

          A list of the types of analysis to perform. Add TABLES to the list to return information
          about the tables that are detected in the input document. Add FORMS to return detected
          form data. To perform both types of analysis, add TABLES and FORMS to ``FeatureTypes`` .
          All lines and words detected in the document are included in the response (including text
          that isn't related to the value of ``FeatureTypes`` ).

          - *(string) --*

        :type HumanLoopConfig: dict
        :param HumanLoopConfig:

          Sets the configuration for the human in the loop workflow for analyzing documents.

          - **HumanLoopName** *(string) --* **[REQUIRED]**

            The name of the human workflow used for this image. This should be kept unique within a
            region.

          - **FlowDefinitionArn** *(string) --* **[REQUIRED]**

            The Amazon Resource Name (ARN) of the flow definition.

          - **DataAttributes** *(dict) --*

            Sets attributes of the input data.

            - **ContentClassifiers** *(list) --*

              Sets whether the input image is free of personally identifiable information or adult
              content.

              - *(string) --*

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'DocumentMetadata': {
                    'Pages': 123
                },
                'Blocks': [
                    {
                        'BlockType':
                        'KEY_VALUE_SET'|'PAGE'|'LINE'|'WORD'|'TABLE'|'CELL'
                        |'SELECTION_ELEMENT',
                        'Confidence': ...,
                        'Text': 'string',
                        'RowIndex': 123,
                        'ColumnIndex': 123,
                        'RowSpan': 123,
                        'ColumnSpan': 123,
                        'Geometry': {
                            'BoundingBox': {
                                'Width': ...,
                                'Height': ...,
                                'Left': ...,
                                'Top': ...
                            },
                            'Polygon': [
                                {
                                    'X': ...,
                                    'Y': ...
                                },
                            ]
                        },
                        'Id': 'string',
                        'Relationships': [
                            {
                                'Type': 'VALUE'|'CHILD',
                                'Ids': [
                                    'string',
                                ]
                            },
                        ],
                        'EntityTypes': [
                            'KEY'|'VALUE',
                        ],
                        'SelectionStatus': 'SELECTED'|'NOT_SELECTED',
                        'Page': 123
                    },
                ],
                'HumanLoopActivationOutput': {
                    'HumanLoopArn': 'string',
                    'HumanLoopActivationReasons': [
                        'string',
                    ],
                    'HumanLoopActivationConditionsEvaluationResults': 'string'
                },
                'AnalyzeDocumentModelVersion': 'string'
            }
          **Response Structure**

          - *(dict) --*

            - **DocumentMetadata** *(dict) --*

              Metadata about the analyzed document. An example is the number of pages.

              - **Pages** *(integer) --*

                The number of pages that are detected in the document.

            - **Blocks** *(list) --*

              The items that are detected and analyzed by ``AnalyzeDocument`` .

              - *(dict) --*

                A ``Block`` represents items that are recognized in a document within a group of
                pixels close to each other. The information returned in a ``Block`` object depends
                on the type of operation. In text detection for documents (for example
                DetectDocumentText ), you get information about the detected words and lines of
                text. In text analysis (for example  AnalyzeDocument ), you can also get information
                about the fields, tables, and selection elements that are detected in the document.

                An array of ``Block`` objects is returned by both synchronous and asynchronous
                operations. In synchronous operations, such as  DetectDocumentText , the array of
                ``Block`` objects is the entire set of results. In asynchronous operations, such as
                GetDocumentAnalysis , the array is returned over one or more responses.

                For more information, see `How Amazon Textract Works
                <https://docs.aws.amazon.com/textract/latest/dg/how-it-works.html>`__ .

                - **BlockType** *(string) --*

                  The type of text item that's recognized. In operations for text detection, the
                  following types are returned:

                  * *PAGE* - Contains a list of the LINE ``Block`` objects that are detected on a
                  document page.

                  * *WORD* - A word detected on a document page. A word is one or more ISO basic
                  Latin script characters that aren't separated by spaces.

                  * *LINE* - A string of tab-delimited, contiguous words that are detected on a
                  document page.

                  In text analysis operations, the following types are returned:

                  * *PAGE* - Contains a list of child ``Block`` objects that are detected on a
                  document page.

                  * *KEY_VALUE_SET* - Stores the KEY and VALUE ``Block`` objects for linked text
                  that's detected on a document page. Use the ``EntityType`` field to determine if a
                  KEY_VALUE_SET object is a KEY ``Block`` object or a VALUE ``Block`` object.

                  * *WORD* - A word that's detected on a document page. A word is one or more ISO
                  basic Latin script characters that aren't separated by spaces.

                  * *LINE* - A string of tab-delimited, contiguous words that are detected on a
                  document page.

                  * *TABLE* - A table that's detected on a document page. A table is grid-based
                  information with two or more rows or columns, with a cell span of one row and one
                  column each.

                  * *CELL* - A cell within a detected table. The cell is the parent of the block
                  that contains the text in the cell.

                  * *SELECTION_ELEMENT* - A selection element such as an option button (radio
                  button) or a check box that's detected on a document page. Use the value of
                  ``SelectionStatus`` to determine the status of the selection element.

                - **Confidence** *(float) --*

                  The confidence score that Amazon Textract has in the accuracy of the recognized
                  text and the accuracy of the geometry points around the recognized text.

                - **Text** *(string) --*

                  The word or line of text that's recognized by Amazon Textract.

                - **RowIndex** *(integer) --*

                  The row in which a table cell is located. The first row position is 1.
                  ``RowIndex`` isn't returned by ``DetectDocumentText`` and
                  ``GetDocumentTextDetection`` .

                - **ColumnIndex** *(integer) --*

                  The column in which a table cell appears. The first column position is 1.
                  ``ColumnIndex`` isn't returned by ``DetectDocumentText`` and
                  ``GetDocumentTextDetection`` .

                - **RowSpan** *(integer) --*

                  The number of rows that a table cell spans. Currently this value is always 1, even
                  if the number of rows spanned is greater than 1. ``RowSpan`` isn't returned by
                  ``DetectDocumentText`` and ``GetDocumentTextDetection`` .

                - **ColumnSpan** *(integer) --*

                  The number of columns that a table cell spans. Currently this value is always 1,
                  even if the number of columns spanned is greater than 1. ``ColumnSpan`` isn't
                  returned by ``DetectDocumentText`` and ``GetDocumentTextDetection`` .

                - **Geometry** *(dict) --*

                  The location of the recognized text on the image. It includes an axis-aligned,
                  coarse bounding box that surrounds the text, and a finer-grain polygon for more
                  accurate spatial information.

                  - **BoundingBox** *(dict) --*

                    An axis-aligned coarse representation of the location of the recognized item on
                    the document page.

                    - **Width** *(float) --*

                      The width of the bounding box as a ratio of the overall document page width.

                    - **Height** *(float) --*

                      The height of the bounding box as a ratio of the overall document page height.

                    - **Left** *(float) --*

                      The left coordinate of the bounding box as a ratio of overall document page
                      width.

                    - **Top** *(float) --*

                      The top coordinate of the bounding box as a ratio of overall document page
                      height.

                  - **Polygon** *(list) --*

                    Within the bounding box, a fine-grained polygon around the recognized item.

                    - *(dict) --*

                      The X and Y coordinates of a point on a document page. The X and Y values that
                      are returned are ratios of the overall document page size. For example, if the
                      input document is 700 x 200 and the operation returns X=
                          0.5 and Y=0.25, then
                      the point is at the (350,50) pixel coordinate on the document page.

                      An array of ``Point`` objects, ``Polygon`` , is returned by
                      DetectDocumentText . ``Polygon`` represents a fine-grained polygon around
                      detected text. For more information, see Geometry in the Amazon Textract
                      Developer Guide.

                      - **X** *(float) --*

                        The value of the X coordinate for a point on a ``Polygon`` .

                      - **Y** *(float) --*

                        The value of the Y coordinate for a point on a ``Polygon`` .

                - **Id** *(string) --*

                  The identifier for the recognized text. The identifier is only unique for a single
                  operation.

                - **Relationships** *(list) --*

                  A list of child blocks of the current block. For example, a LINE object has child
                  blocks for each WORD block that's part of the line of text. There aren't
                  Relationship objects in the list for relationships that don't exist, such as when
                  the current block has no child blocks. The list size can be the following:

                  * 0 - The block has no child blocks.

                  * 1 - The block has child blocks.

                  - *(dict) --*

                    Information about how blocks are related to each other. A ``Block`` object
                    contains 0 or more ``Relation`` objects in a list, ``Relationships`` . For more
                    information, see  Block .

                    The ``Type`` element provides the type of the relationship for all blocks in the
                    ``IDs`` array.

                    - **Type** *(string) --*

                      The type of relationship that the blocks in the IDs array have with the
                      current block. The relationship can be ``VALUE`` or ``CHILD`` . A relationship
                      of type VALUE is a list that contains the ID of the VALUE block that's
                      associated with the KEY of a key-value pair. A relationship of type CHILD is a
                      list of IDs that identify WORD blocks.

                    - **Ids** *(list) --*

                      An array of IDs for related blocks. You can get the type of the relationship
                      from the ``Type`` element.

                      - *(string) --*

                - **EntityTypes** *(list) --*

                  The type of entity. The following can be returned:

                  * *KEY* - An identifier for a field on the document.

                  * *VALUE* - The field text.

                   ``EntityTypes`` isn't returned by ``DetectDocumentText`` and
                   ``GetDocumentTextDetection`` .

                  - *(string) --*

                - **SelectionStatus** *(string) --*

                  The selection status of a selection element, such as an option button or check
                  box.

                - **Page** *(integer) --*

                  The page on which a block was detected. ``Page`` is returned by asynchronous
                  operations. Page values greater than 1 are only returned for multipage documents
                  that are in PDF format. A scanned image (JPEG/PNG), even if it contains multiple
                  document pages, is considered to be a single-page document. The value of ``Page``
                  is always 1. Synchronous operations don't return ``Page`` because every input
                  document is considered to be a single-page document.

            - **HumanLoopActivationOutput** *(dict) --*

              Shows the results of the human in the loop evaluation.

              - **HumanLoopArn** *(string) --*

                The Amazon Resource Name (ARN) of the HumanLoop created.

              - **HumanLoopActivationReasons** *(list) --*

                Shows if and why human review was needed.

                - *(string) --*

              - **HumanLoopActivationConditionsEvaluationResults** *(string) --*

                Shows the result of condition evaluations, including those conditions which
                activated a human review.

            - **AnalyzeDocumentModelVersion** *(string) --*

              The version of the model used to analyze the document.
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def can_paginate(self, operation_name: str) -> bool:
        """
        Check if an operation can be paginated.

        :type operation_name: string
        :param operation_name: The operation name.  This is the same name
            as the method name on the client.  For example, if the
            method name is ``create_foo``, and you'd normally invoke the
            operation as ``client.create_foo(**kwargs)``, if the
            ``create_foo`` operation can be paginated, you can use the
            call ``client.get_paginator("create_foo")``.

        :return: ``True`` if the operation can be paginated,
            ``False`` otherwise.
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def detect_document_text(
        self, Document: ClientDetectDocumentTextDocumentTypeDef
    ) -> ClientDetectDocumentTextResponseTypeDef:
        """
        Detects text in the input document. Amazon Textract can detect lines of text and the words
        that make up a line of text. The input document must be an image in JPEG or PNG format.
        ``DetectDocumentText`` returns the detected text in an array of  Block objects.

        Each document page has as an associated ``Block`` of type PAGE. Each PAGE ``Block`` object
        is the parent of LINE ``Block`` objects that represent the lines of detected text on a page.
        A LINE ``Block`` object is a parent for each word that makes up the line. Words are
        represented by ``Block`` objects of type WORD.

         ``DetectDocumentText`` is a synchronous operation. To analyze documents asynchronously, use
         StartDocumentTextDetection .

        For more information, see `Document Text Detection
        <https://docs.aws.amazon.com/textract/latest/dg/how-it-works-detecting.html>`__ .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/textract-2018-06-27/DetectDocumentText>`_

        **Request Syntax**
        ::

          response = client.detect_document_text(
              Document={
                  'Bytes': b'bytes',
                  'S3Object': {
                      'Bucket': 'string',
                      'Name': 'string',
                      'Version': 'string'
                  }
              }
          )
        :type Document: dict
        :param Document: **[REQUIRED]**

          The input document as base64-encoded bytes or an Amazon S3 object. If you use the AWS CLI
          to call Amazon Textract operations, you can't pass image bytes. The document must be an
          image in JPEG or PNG format.

          If you're using an AWS SDK to call Amazon Textract, you might not need to base64-encode
          image bytes that are passed using the ``Bytes`` field.

          - **Bytes** *(bytes) --*

            A blob of base64-encoded document bytes. The maximum size of a document that's provided
            in a blob of bytes is 5 MB. The document bytes must be in PNG or JPEG format.

            If you're using an AWS SDK to call Amazon Textract, you might not need to base64-encode
            image bytes passed using the ``Bytes`` field.

          - **S3Object** *(dict) --*

            Identifies an S3 object as the document source. The maximum size of a document that's
            stored in an S3 bucket is 5 MB.

            - **Bucket** *(string) --*

              The name of the S3 bucket.

            - **Name** *(string) --*

              The file name of the input document. Synchronous operations can use image files that
              are in JPEG or PNG format. Asynchronous operations also support PDF format files.

            - **Version** *(string) --*

              If the bucket has versioning enabled, you can specify the object version.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'DocumentMetadata': {
                    'Pages': 123
                },
                'Blocks': [
                    {
                        'BlockType':
                        'KEY_VALUE_SET'|'PAGE'|'LINE'|'WORD'|'TABLE'|'CELL'
                        |'SELECTION_ELEMENT',
                        'Confidence': ...,
                        'Text': 'string',
                        'RowIndex': 123,
                        'ColumnIndex': 123,
                        'RowSpan': 123,
                        'ColumnSpan': 123,
                        'Geometry': {
                            'BoundingBox': {
                                'Width': ...,
                                'Height': ...,
                                'Left': ...,
                                'Top': ...
                            },
                            'Polygon': [
                                {
                                    'X': ...,
                                    'Y': ...
                                },
                            ]
                        },
                        'Id': 'string',
                        'Relationships': [
                            {
                                'Type': 'VALUE'|'CHILD',
                                'Ids': [
                                    'string',
                                ]
                            },
                        ],
                        'EntityTypes': [
                            'KEY'|'VALUE',
                        ],
                        'SelectionStatus': 'SELECTED'|'NOT_SELECTED',
                        'Page': 123
                    },
                ],
                'DetectDocumentTextModelVersion': 'string'
            }
          **Response Structure**

          - *(dict) --*

            - **DocumentMetadata** *(dict) --*

              Metadata about the document. It contains the number of pages that are detected in the
              document.

              - **Pages** *(integer) --*

                The number of pages that are detected in the document.

            - **Blocks** *(list) --*

              An array of ``Block`` objects that contain the text that's detected in the document.

              - *(dict) --*

                A ``Block`` represents items that are recognized in a document within a group of
                pixels close to each other. The information returned in a ``Block`` object depends
                on the type of operation. In text detection for documents (for example
                DetectDocumentText ), you get information about the detected words and lines of
                text. In text analysis (for example  AnalyzeDocument ), you can also get information
                about the fields, tables, and selection elements that are detected in the document.

                An array of ``Block`` objects is returned by both synchronous and asynchronous
                operations. In synchronous operations, such as  DetectDocumentText , the array of
                ``Block`` objects is the entire set of results. In asynchronous operations, such as
                GetDocumentAnalysis , the array is returned over one or more responses.

                For more information, see `How Amazon Textract Works
                <https://docs.aws.amazon.com/textract/latest/dg/how-it-works.html>`__ .

                - **BlockType** *(string) --*

                  The type of text item that's recognized. In operations for text detection, the
                  following types are returned:

                  * *PAGE* - Contains a list of the LINE ``Block`` objects that are detected on a
                  document page.

                  * *WORD* - A word detected on a document page. A word is one or more ISO basic
                  Latin script characters that aren't separated by spaces.

                  * *LINE* - A string of tab-delimited, contiguous words that are detected on a
                  document page.

                  In text analysis operations, the following types are returned:

                  * *PAGE* - Contains a list of child ``Block`` objects that are detected on a
                  document page.

                  * *KEY_VALUE_SET* - Stores the KEY and VALUE ``Block`` objects for linked text
                  that's detected on a document page. Use the ``EntityType`` field to determine if a
                  KEY_VALUE_SET object is a KEY ``Block`` object or a VALUE ``Block`` object.

                  * *WORD* - A word that's detected on a document page. A word is one or more ISO
                  basic Latin script characters that aren't separated by spaces.

                  * *LINE* - A string of tab-delimited, contiguous words that are detected on a
                  document page.

                  * *TABLE* - A table that's detected on a document page. A table is grid-based
                  information with two or more rows or columns, with a cell span of one row and one
                  column each.

                  * *CELL* - A cell within a detected table. The cell is the parent of the block
                  that contains the text in the cell.

                  * *SELECTION_ELEMENT* - A selection element such as an option button (radio
                  button) or a check box that's detected on a document page. Use the value of
                  ``SelectionStatus`` to determine the status of the selection element.

                - **Confidence** *(float) --*

                  The confidence score that Amazon Textract has in the accuracy of the recognized
                  text and the accuracy of the geometry points around the recognized text.

                - **Text** *(string) --*

                  The word or line of text that's recognized by Amazon Textract.

                - **RowIndex** *(integer) --*

                  The row in which a table cell is located. The first row position is 1.
                  ``RowIndex`` isn't returned by ``DetectDocumentText`` and
                  ``GetDocumentTextDetection`` .

                - **ColumnIndex** *(integer) --*

                  The column in which a table cell appears. The first column position is 1.
                  ``ColumnIndex`` isn't returned by ``DetectDocumentText`` and
                  ``GetDocumentTextDetection`` .

                - **RowSpan** *(integer) --*

                  The number of rows that a table cell spans. Currently this value is always 1, even
                  if the number of rows spanned is greater than 1. ``RowSpan`` isn't returned by
                  ``DetectDocumentText`` and ``GetDocumentTextDetection`` .

                - **ColumnSpan** *(integer) --*

                  The number of columns that a table cell spans. Currently this value is always 1,
                  even if the number of columns spanned is greater than 1. ``ColumnSpan`` isn't
                  returned by ``DetectDocumentText`` and ``GetDocumentTextDetection`` .

                - **Geometry** *(dict) --*

                  The location of the recognized text on the image. It includes an axis-aligned,
                  coarse bounding box that surrounds the text, and a finer-grain polygon for more
                  accurate spatial information.

                  - **BoundingBox** *(dict) --*

                    An axis-aligned coarse representation of the location of the recognized item on
                    the document page.

                    - **Width** *(float) --*

                      The width of the bounding box as a ratio of the overall document page width.

                    - **Height** *(float) --*

                      The height of the bounding box as a ratio of the overall document page height.

                    - **Left** *(float) --*

                      The left coordinate of the bounding box as a ratio of overall document page
                      width.

                    - **Top** *(float) --*

                      The top coordinate of the bounding box as a ratio of overall document page
                      height.

                  - **Polygon** *(list) --*

                    Within the bounding box, a fine-grained polygon around the recognized item.

                    - *(dict) --*

                      The X and Y coordinates of a point on a document page. The X and Y values that
                      are returned are ratios of the overall document page size. For example, if the
                      input document is 700 x 200 and the operation returns X=
                          0.5 and Y=0.25, then
                      the point is at the (350,50) pixel coordinate on the document page.

                      An array of ``Point`` objects, ``Polygon`` , is returned by
                      DetectDocumentText . ``Polygon`` represents a fine-grained polygon around
                      detected text. For more information, see Geometry in the Amazon Textract
                      Developer Guide.

                      - **X** *(float) --*

                        The value of the X coordinate for a point on a ``Polygon`` .

                      - **Y** *(float) --*

                        The value of the Y coordinate for a point on a ``Polygon`` .

                - **Id** *(string) --*

                  The identifier for the recognized text. The identifier is only unique for a single
                  operation.

                - **Relationships** *(list) --*

                  A list of child blocks of the current block. For example, a LINE object has child
                  blocks for each WORD block that's part of the line of text. There aren't
                  Relationship objects in the list for relationships that don't exist, such as when
                  the current block has no child blocks. The list size can be the following:

                  * 0 - The block has no child blocks.

                  * 1 - The block has child blocks.

                  - *(dict) --*

                    Information about how blocks are related to each other. A ``Block`` object
                    contains 0 or more ``Relation`` objects in a list, ``Relationships`` . For more
                    information, see  Block .

                    The ``Type`` element provides the type of the relationship for all blocks in the
                    ``IDs`` array.

                    - **Type** *(string) --*

                      The type of relationship that the blocks in the IDs array have with the
                      current block. The relationship can be ``VALUE`` or ``CHILD`` . A relationship
                      of type VALUE is a list that contains the ID of the VALUE block that's
                      associated with the KEY of a key-value pair. A relationship of type CHILD is a
                      list of IDs that identify WORD blocks.

                    - **Ids** *(list) --*

                      An array of IDs for related blocks. You can get the type of the relationship
                      from the ``Type`` element.

                      - *(string) --*

                - **EntityTypes** *(list) --*

                  The type of entity. The following can be returned:

                  * *KEY* - An identifier for a field on the document.

                  * *VALUE* - The field text.

                   ``EntityTypes`` isn't returned by ``DetectDocumentText`` and
                   ``GetDocumentTextDetection`` .

                  - *(string) --*

                - **SelectionStatus** *(string) --*

                  The selection status of a selection element, such as an option button or check
                  box.

                - **Page** *(integer) --*

                  The page on which a block was detected. ``Page`` is returned by asynchronous
                  operations. Page values greater than 1 are only returned for multipage documents
                  that are in PDF format. A scanned image (JPEG/PNG), even if it contains multiple
                  document pages, is considered to be a single-page document. The value of ``Page``
                  is always 1. Synchronous operations don't return ``Page`` because every input
                  document is considered to be a single-page document.

            - **DetectDocumentTextModelVersion** *(string) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def generate_presigned_url(
        self,
        ClientMethod: str,
        Params: Dict[str, Any] = None,
        ExpiresIn: int = 3600,
        HttpMethod: str = None,
    ) -> None:
        """
        Generate a presigned url given a client, its method, and arguments

        :type ClientMethod: string
        :param ClientMethod: The client method to presign for

        :type Params: dict
        :param Params: The parameters normally passed to
            ``ClientMethod``.

        :type ExpiresIn: int
        :param ExpiresIn: The number of seconds the presigned url is valid
            for. By default it expires in an hour (3600 seconds)

        :type HttpMethod: string
        :param HttpMethod: The http method to use on the generated url. By
            default, the http method is whatever is used in the method's model.

        :returns: The presigned url
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_document_analysis(
        self, JobId: str, MaxResults: int = None, NextToken: str = None
    ) -> ClientGetDocumentAnalysisResponseTypeDef:
        """
        Gets the results for an Amazon Textract asynchronous operation that analyzes text in a
        document.

        You start asynchronous text analysis by calling  StartDocumentAnalysis , which returns a job
        identifier (``JobId`` ). When the text analysis operation finishes, Amazon Textract
        publishes a completion status to the Amazon Simple Notification Service (Amazon SNS) topic
        that's registered in the initial call to ``StartDocumentAnalysis`` . To get the results of
        the text-detection operation, first check that the status value published to the Amazon SNS
        topic is ``SUCCEEDED`` . If so, call ``GetDocumentAnalysis`` , and pass the job identifier
        (``JobId`` ) from the initial call to ``StartDocumentAnalysis`` .

         ``GetDocumentAnalysis`` returns an array of  Block objects. The following types of
         information are returned:

        * Form data (key-value pairs). The related information is returned in two  Block objects,
        each of type ``KEY_VALUE_SET`` : a KEY ``Block`` object and a VALUE ``Block`` object. For
        example, *Name: Ana Silva Carolina* contains a key and value. *Name:* is the key. *Ana Silva
        Carolina* is the value.

        * Table and table cell data. A TABLE ``Block`` object contains information about a detected
        table. A CELL ``Block`` object is returned for each cell in a table.

        * Lines and words of text. A LINE ``Block`` object contains one or more WORD ``Block``
        objects. All lines and words that are detected in the document are returned (including text
        that doesn't have a relationship with the value of the ``StartDocumentAnalysis``
        ``FeatureTypes`` input parameter).

        Selection elements such as check boxes and option buttons (radio buttons) can be detected in
        form data and in tables. A SELECTION_ELEMENT ``Block`` object contains information about a
        selection element, including the selection status.

        Use the ``MaxResults`` parameter to limit the number of blocks that are returned. If there
        are more results than specified in ``MaxResults`` , the value of ``NextToken`` in the
        operation response contains a pagination token for getting the next set of results. To get
        the next page of results, call ``GetDocumentAnalysis`` , and populate the ``NextToken``
        request parameter with the token value that's returned from the previous call to
        ``GetDocumentAnalysis`` .

        For more information, see `Document Text Analysis
        <https://docs.aws.amazon.com/textract/latest/dg/how-it-works-analyzing.html>`__ .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/textract-2018-06-27/GetDocumentAnalysis>`_

        **Request Syntax**
        ::

          response = client.get_document_analysis(
              JobId='string',
              MaxResults=123,
              NextToken='string'
          )
        :type JobId: string
        :param JobId: **[REQUIRED]**

          A unique identifier for the text-detection job. The ``JobId`` is returned from
          ``StartDocumentAnalysis`` . A ``JobId`` value is only valid for 7 days.

        :type MaxResults: integer
        :param MaxResults:

          The maximum number of results to return per paginated call. The largest value that you can
          specify is 1,000. If you specify a value greater than 1,000, a maximum of 1,000 results is
          returned. The default value is 1,000.

        :type NextToken: string
        :param NextToken:

          If the previous response was incomplete (because there are more blocks to retrieve),
          Amazon Textract returns a pagination token in the response. You can use this pagination
          token to retrieve the next set of blocks.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'DocumentMetadata': {
                    'Pages': 123
                },
                'JobStatus': 'IN_PROGRESS'|'SUCCEEDED'|'FAILED'|'PARTIAL_SUCCESS',
                'NextToken': 'string',
                'Blocks': [
                    {
                        'BlockType':
                        'KEY_VALUE_SET'|'PAGE'|'LINE'|'WORD'|'TABLE'|'CELL'
                        |'SELECTION_ELEMENT',
                        'Confidence': ...,
                        'Text': 'string',
                        'RowIndex': 123,
                        'ColumnIndex': 123,
                        'RowSpan': 123,
                        'ColumnSpan': 123,
                        'Geometry': {
                            'BoundingBox': {
                                'Width': ...,
                                'Height': ...,
                                'Left': ...,
                                'Top': ...
                            },
                            'Polygon': [
                                {
                                    'X': ...,
                                    'Y': ...
                                },
                            ]
                        },
                        'Id': 'string',
                        'Relationships': [
                            {
                                'Type': 'VALUE'|'CHILD',
                                'Ids': [
                                    'string',
                                ]
                            },
                        ],
                        'EntityTypes': [
                            'KEY'|'VALUE',
                        ],
                        'SelectionStatus': 'SELECTED'|'NOT_SELECTED',
                        'Page': 123
                    },
                ],
                'Warnings': [
                    {
                        'ErrorCode': 'string',
                        'Pages': [
                            123,
                        ]
                    },
                ],
                'StatusMessage': 'string',
                'AnalyzeDocumentModelVersion': 'string'
            }
          **Response Structure**

          - *(dict) --*

            - **DocumentMetadata** *(dict) --*

              Information about a document that Amazon Textract processed. ``DocumentMetadata`` is
              returned in every page of paginated responses from an Amazon Textract video operation.

              - **Pages** *(integer) --*

                The number of pages that are detected in the document.

            - **JobStatus** *(string) --*

              The current status of the text detection job.

            - **NextToken** *(string) --*

              If the response is truncated, Amazon Textract returns this token. You can use this
              token in the subsequent request to retrieve the next set of text detection results.

            - **Blocks** *(list) --*

              The results of the text-analysis operation.

              - *(dict) --*

                A ``Block`` represents items that are recognized in a document within a group of
                pixels close to each other. The information returned in a ``Block`` object depends
                on the type of operation. In text detection for documents (for example
                DetectDocumentText ), you get information about the detected words and lines of
                text. In text analysis (for example  AnalyzeDocument ), you can also get information
                about the fields, tables, and selection elements that are detected in the document.

                An array of ``Block`` objects is returned by both synchronous and asynchronous
                operations. In synchronous operations, such as  DetectDocumentText , the array of
                ``Block`` objects is the entire set of results. In asynchronous operations, such as
                GetDocumentAnalysis , the array is returned over one or more responses.

                For more information, see `How Amazon Textract Works
                <https://docs.aws.amazon.com/textract/latest/dg/how-it-works.html>`__ .

                - **BlockType** *(string) --*

                  The type of text item that's recognized. In operations for text detection, the
                  following types are returned:

                  * *PAGE* - Contains a list of the LINE ``Block`` objects that are detected on a
                  document page.

                  * *WORD* - A word detected on a document page. A word is one or more ISO basic
                  Latin script characters that aren't separated by spaces.

                  * *LINE* - A string of tab-delimited, contiguous words that are detected on a
                  document page.

                  In text analysis operations, the following types are returned:

                  * *PAGE* - Contains a list of child ``Block`` objects that are detected on a
                  document page.

                  * *KEY_VALUE_SET* - Stores the KEY and VALUE ``Block`` objects for linked text
                  that's detected on a document page. Use the ``EntityType`` field to determine if a
                  KEY_VALUE_SET object is a KEY ``Block`` object or a VALUE ``Block`` object.

                  * *WORD* - A word that's detected on a document page. A word is one or more ISO
                  basic Latin script characters that aren't separated by spaces.

                  * *LINE* - A string of tab-delimited, contiguous words that are detected on a
                  document page.

                  * *TABLE* - A table that's detected on a document page. A table is grid-based
                  information with two or more rows or columns, with a cell span of one row and one
                  column each.

                  * *CELL* - A cell within a detected table. The cell is the parent of the block
                  that contains the text in the cell.

                  * *SELECTION_ELEMENT* - A selection element such as an option button (radio
                  button) or a check box that's detected on a document page. Use the value of
                  ``SelectionStatus`` to determine the status of the selection element.

                - **Confidence** *(float) --*

                  The confidence score that Amazon Textract has in the accuracy of the recognized
                  text and the accuracy of the geometry points around the recognized text.

                - **Text** *(string) --*

                  The word or line of text that's recognized by Amazon Textract.

                - **RowIndex** *(integer) --*

                  The row in which a table cell is located. The first row position is 1.
                  ``RowIndex`` isn't returned by ``DetectDocumentText`` and
                  ``GetDocumentTextDetection`` .

                - **ColumnIndex** *(integer) --*

                  The column in which a table cell appears. The first column position is 1.
                  ``ColumnIndex`` isn't returned by ``DetectDocumentText`` and
                  ``GetDocumentTextDetection`` .

                - **RowSpan** *(integer) --*

                  The number of rows that a table cell spans. Currently this value is always 1, even
                  if the number of rows spanned is greater than 1. ``RowSpan`` isn't returned by
                  ``DetectDocumentText`` and ``GetDocumentTextDetection`` .

                - **ColumnSpan** *(integer) --*

                  The number of columns that a table cell spans. Currently this value is always 1,
                  even if the number of columns spanned is greater than 1. ``ColumnSpan`` isn't
                  returned by ``DetectDocumentText`` and ``GetDocumentTextDetection`` .

                - **Geometry** *(dict) --*

                  The location of the recognized text on the image. It includes an axis-aligned,
                  coarse bounding box that surrounds the text, and a finer-grain polygon for more
                  accurate spatial information.

                  - **BoundingBox** *(dict) --*

                    An axis-aligned coarse representation of the location of the recognized item on
                    the document page.

                    - **Width** *(float) --*

                      The width of the bounding box as a ratio of the overall document page width.

                    - **Height** *(float) --*

                      The height of the bounding box as a ratio of the overall document page height.

                    - **Left** *(float) --*

                      The left coordinate of the bounding box as a ratio of overall document page
                      width.

                    - **Top** *(float) --*

                      The top coordinate of the bounding box as a ratio of overall document page
                      height.

                  - **Polygon** *(list) --*

                    Within the bounding box, a fine-grained polygon around the recognized item.

                    - *(dict) --*

                      The X and Y coordinates of a point on a document page. The X and Y values that
                      are returned are ratios of the overall document page size. For example, if the
                      input document is 700 x 200 and the operation returns X=
                          0.5 and Y=0.25, then
                      the point is at the (350,50) pixel coordinate on the document page.

                      An array of ``Point`` objects, ``Polygon`` , is returned by
                      DetectDocumentText . ``Polygon`` represents a fine-grained polygon around
                      detected text. For more information, see Geometry in the Amazon Textract
                      Developer Guide.

                      - **X** *(float) --*

                        The value of the X coordinate for a point on a ``Polygon`` .

                      - **Y** *(float) --*

                        The value of the Y coordinate for a point on a ``Polygon`` .

                - **Id** *(string) --*

                  The identifier for the recognized text. The identifier is only unique for a single
                  operation.

                - **Relationships** *(list) --*

                  A list of child blocks of the current block. For example, a LINE object has child
                  blocks for each WORD block that's part of the line of text. There aren't
                  Relationship objects in the list for relationships that don't exist, such as when
                  the current block has no child blocks. The list size can be the following:

                  * 0 - The block has no child blocks.

                  * 1 - The block has child blocks.

                  - *(dict) --*

                    Information about how blocks are related to each other. A ``Block`` object
                    contains 0 or more ``Relation`` objects in a list, ``Relationships`` . For more
                    information, see  Block .

                    The ``Type`` element provides the type of the relationship for all blocks in the
                    ``IDs`` array.

                    - **Type** *(string) --*

                      The type of relationship that the blocks in the IDs array have with the
                      current block. The relationship can be ``VALUE`` or ``CHILD`` . A relationship
                      of type VALUE is a list that contains the ID of the VALUE block that's
                      associated with the KEY of a key-value pair. A relationship of type CHILD is a
                      list of IDs that identify WORD blocks.

                    - **Ids** *(list) --*

                      An array of IDs for related blocks. You can get the type of the relationship
                      from the ``Type`` element.

                      - *(string) --*

                - **EntityTypes** *(list) --*

                  The type of entity. The following can be returned:

                  * *KEY* - An identifier for a field on the document.

                  * *VALUE* - The field text.

                   ``EntityTypes`` isn't returned by ``DetectDocumentText`` and
                   ``GetDocumentTextDetection`` .

                  - *(string) --*

                - **SelectionStatus** *(string) --*

                  The selection status of a selection element, such as an option button or check
                  box.

                - **Page** *(integer) --*

                  The page on which a block was detected. ``Page`` is returned by asynchronous
                  operations. Page values greater than 1 are only returned for multipage documents
                  that are in PDF format. A scanned image (JPEG/PNG), even if it contains multiple
                  document pages, is considered to be a single-page document. The value of ``Page``
                  is always 1. Synchronous operations don't return ``Page`` because every input
                  document is considered to be a single-page document.

            - **Warnings** *(list) --*

              A list of warnings that occurred during the document-analysis operation.

              - *(dict) --*

                A warning about an issue that occurred during asynchronous text analysis (
                StartDocumentAnalysis ) or asynchronous document text detection (
                StartDocumentTextDetection ).

                - **ErrorCode** *(string) --*

                  The error code for the warning.

                - **Pages** *(list) --*

                  A list of the pages that the warning applies to.

                  - *(integer) --*

            - **StatusMessage** *(string) --*

              The current status of an asynchronous document-analysis operation.

            - **AnalyzeDocumentModelVersion** *(string) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_document_text_detection(
        self, JobId: str, MaxResults: int = None, NextToken: str = None
    ) -> ClientGetDocumentTextDetectionResponseTypeDef:
        """
        Gets the results for an Amazon Textract asynchronous operation that detects text in a
        document. Amazon Textract can detect lines of text and the words that make up a line of
        text.

        You start asynchronous text detection by calling  StartDocumentTextDetection , which returns
        a job identifier (``JobId`` ). When the text detection operation finishes, Amazon Textract
        publishes a completion status to the Amazon Simple Notification Service (Amazon SNS) topic
        that's registered in the initial call to ``StartDocumentTextDetection`` . To get the results
        of the text-detection operation, first check that the status value published to the Amazon
        SNS topic is ``SUCCEEDED`` . If so, call ``GetDocumentTextDetection`` , and pass the job
        identifier (``JobId`` ) from the initial call to ``StartDocumentTextDetection`` .

         ``GetDocumentTextDetection`` returns an array of  Block objects.

        Each document page has as an associated ``Block`` of type PAGE. Each PAGE ``Block`` object
        is the parent of LINE ``Block`` objects that represent the lines of detected text on a page.
        A LINE ``Block`` object is a parent for each word that makes up the line. Words are
        represented by ``Block`` objects of type WORD.

        Use the MaxResults parameter to limit the number of blocks that are returned. If there are
        more results than specified in ``MaxResults`` , the value of ``NextToken`` in the operation
        response contains a pagination token for getting the next set of results. To get the next
        page of results, call ``GetDocumentTextDetection`` , and populate the ``NextToken`` request
        parameter with the token value that's returned from the previous call to
        ``GetDocumentTextDetection`` .

        For more information, see `Document Text Detection
        <https://docs.aws.amazon.com/textract/latest/dg/how-it-works-detecting.html>`__ .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/textract-2018-06-27/GetDocumentTextDetection>`_

        **Request Syntax**
        ::

          response = client.get_document_text_detection(
              JobId='string',
              MaxResults=123,
              NextToken='string'
          )
        :type JobId: string
        :param JobId: **[REQUIRED]**

          A unique identifier for the text detection job. The ``JobId`` is returned from
          ``StartDocumentTextDetection`` . A ``JobId`` value is only valid for 7 days.

        :type MaxResults: integer
        :param MaxResults:

          The maximum number of results to return per paginated call. The largest value you can
          specify is 1,000. If you specify a value greater than 1,000, a maximum of 1,000 results is
          returned. The default value is 1,000.

        :type NextToken: string
        :param NextToken:

          If the previous response was incomplete (because there are more blocks to retrieve),
          Amazon Textract returns a pagination token in the response. You can use this pagination
          token to retrieve the next set of blocks.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'DocumentMetadata': {
                    'Pages': 123
                },
                'JobStatus': 'IN_PROGRESS'|'SUCCEEDED'|'FAILED'|'PARTIAL_SUCCESS',
                'NextToken': 'string',
                'Blocks': [
                    {
                        'BlockType':
                        'KEY_VALUE_SET'|'PAGE'|'LINE'|'WORD'|'TABLE'|'CELL'
                        |'SELECTION_ELEMENT',
                        'Confidence': ...,
                        'Text': 'string',
                        'RowIndex': 123,
                        'ColumnIndex': 123,
                        'RowSpan': 123,
                        'ColumnSpan': 123,
                        'Geometry': {
                            'BoundingBox': {
                                'Width': ...,
                                'Height': ...,
                                'Left': ...,
                                'Top': ...
                            },
                            'Polygon': [
                                {
                                    'X': ...,
                                    'Y': ...
                                },
                            ]
                        },
                        'Id': 'string',
                        'Relationships': [
                            {
                                'Type': 'VALUE'|'CHILD',
                                'Ids': [
                                    'string',
                                ]
                            },
                        ],
                        'EntityTypes': [
                            'KEY'|'VALUE',
                        ],
                        'SelectionStatus': 'SELECTED'|'NOT_SELECTED',
                        'Page': 123
                    },
                ],
                'Warnings': [
                    {
                        'ErrorCode': 'string',
                        'Pages': [
                            123,
                        ]
                    },
                ],
                'StatusMessage': 'string',
                'DetectDocumentTextModelVersion': 'string'
            }
          **Response Structure**

          - *(dict) --*

            - **DocumentMetadata** *(dict) --*

              Information about a document that Amazon Textract processed. ``DocumentMetadata`` is
              returned in every page of paginated responses from an Amazon Textract video operation.

              - **Pages** *(integer) --*

                The number of pages that are detected in the document.

            - **JobStatus** *(string) --*

              The current status of the text detection job.

            - **NextToken** *(string) --*

              If the response is truncated, Amazon Textract returns this token. You can use this
              token in the subsequent request to retrieve the next set of text-detection results.

            - **Blocks** *(list) --*

              The results of the text-detection operation.

              - *(dict) --*

                A ``Block`` represents items that are recognized in a document within a group of
                pixels close to each other. The information returned in a ``Block`` object depends
                on the type of operation. In text detection for documents (for example
                DetectDocumentText ), you get information about the detected words and lines of
                text. In text analysis (for example  AnalyzeDocument ), you can also get information
                about the fields, tables, and selection elements that are detected in the document.

                An array of ``Block`` objects is returned by both synchronous and asynchronous
                operations. In synchronous operations, such as  DetectDocumentText , the array of
                ``Block`` objects is the entire set of results. In asynchronous operations, such as
                GetDocumentAnalysis , the array is returned over one or more responses.

                For more information, see `How Amazon Textract Works
                <https://docs.aws.amazon.com/textract/latest/dg/how-it-works.html>`__ .

                - **BlockType** *(string) --*

                  The type of text item that's recognized. In operations for text detection, the
                  following types are returned:

                  * *PAGE* - Contains a list of the LINE ``Block`` objects that are detected on a
                  document page.

                  * *WORD* - A word detected on a document page. A word is one or more ISO basic
                  Latin script characters that aren't separated by spaces.

                  * *LINE* - A string of tab-delimited, contiguous words that are detected on a
                  document page.

                  In text analysis operations, the following types are returned:

                  * *PAGE* - Contains a list of child ``Block`` objects that are detected on a
                  document page.

                  * *KEY_VALUE_SET* - Stores the KEY and VALUE ``Block`` objects for linked text
                  that's detected on a document page. Use the ``EntityType`` field to determine if a
                  KEY_VALUE_SET object is a KEY ``Block`` object or a VALUE ``Block`` object.

                  * *WORD* - A word that's detected on a document page. A word is one or more ISO
                  basic Latin script characters that aren't separated by spaces.

                  * *LINE* - A string of tab-delimited, contiguous words that are detected on a
                  document page.

                  * *TABLE* - A table that's detected on a document page. A table is grid-based
                  information with two or more rows or columns, with a cell span of one row and one
                  column each.

                  * *CELL* - A cell within a detected table. The cell is the parent of the block
                  that contains the text in the cell.

                  * *SELECTION_ELEMENT* - A selection element such as an option button (radio
                  button) or a check box that's detected on a document page. Use the value of
                  ``SelectionStatus`` to determine the status of the selection element.

                - **Confidence** *(float) --*

                  The confidence score that Amazon Textract has in the accuracy of the recognized
                  text and the accuracy of the geometry points around the recognized text.

                - **Text** *(string) --*

                  The word or line of text that's recognized by Amazon Textract.

                - **RowIndex** *(integer) --*

                  The row in which a table cell is located. The first row position is 1.
                  ``RowIndex`` isn't returned by ``DetectDocumentText`` and
                  ``GetDocumentTextDetection`` .

                - **ColumnIndex** *(integer) --*

                  The column in which a table cell appears. The first column position is 1.
                  ``ColumnIndex`` isn't returned by ``DetectDocumentText`` and
                  ``GetDocumentTextDetection`` .

                - **RowSpan** *(integer) --*

                  The number of rows that a table cell spans. Currently this value is always 1, even
                  if the number of rows spanned is greater than 1. ``RowSpan`` isn't returned by
                  ``DetectDocumentText`` and ``GetDocumentTextDetection`` .

                - **ColumnSpan** *(integer) --*

                  The number of columns that a table cell spans. Currently this value is always 1,
                  even if the number of columns spanned is greater than 1. ``ColumnSpan`` isn't
                  returned by ``DetectDocumentText`` and ``GetDocumentTextDetection`` .

                - **Geometry** *(dict) --*

                  The location of the recognized text on the image. It includes an axis-aligned,
                  coarse bounding box that surrounds the text, and a finer-grain polygon for more
                  accurate spatial information.

                  - **BoundingBox** *(dict) --*

                    An axis-aligned coarse representation of the location of the recognized item on
                    the document page.

                    - **Width** *(float) --*

                      The width of the bounding box as a ratio of the overall document page width.

                    - **Height** *(float) --*

                      The height of the bounding box as a ratio of the overall document page height.

                    - **Left** *(float) --*

                      The left coordinate of the bounding box as a ratio of overall document page
                      width.

                    - **Top** *(float) --*

                      The top coordinate of the bounding box as a ratio of overall document page
                      height.

                  - **Polygon** *(list) --*

                    Within the bounding box, a fine-grained polygon around the recognized item.

                    - *(dict) --*

                      The X and Y coordinates of a point on a document page. The X and Y values that
                      are returned are ratios of the overall document page size. For example, if the
                      input document is 700 x 200 and the operation returns X=
                          0.5 and Y=0.25, then
                      the point is at the (350,50) pixel coordinate on the document page.

                      An array of ``Point`` objects, ``Polygon`` , is returned by
                      DetectDocumentText . ``Polygon`` represents a fine-grained polygon around
                      detected text. For more information, see Geometry in the Amazon Textract
                      Developer Guide.

                      - **X** *(float) --*

                        The value of the X coordinate for a point on a ``Polygon`` .

                      - **Y** *(float) --*

                        The value of the Y coordinate for a point on a ``Polygon`` .

                - **Id** *(string) --*

                  The identifier for the recognized text. The identifier is only unique for a single
                  operation.

                - **Relationships** *(list) --*

                  A list of child blocks of the current block. For example, a LINE object has child
                  blocks for each WORD block that's part of the line of text. There aren't
                  Relationship objects in the list for relationships that don't exist, such as when
                  the current block has no child blocks. The list size can be the following:

                  * 0 - The block has no child blocks.

                  * 1 - The block has child blocks.

                  - *(dict) --*

                    Information about how blocks are related to each other. A ``Block`` object
                    contains 0 or more ``Relation`` objects in a list, ``Relationships`` . For more
                    information, see  Block .

                    The ``Type`` element provides the type of the relationship for all blocks in the
                    ``IDs`` array.

                    - **Type** *(string) --*

                      The type of relationship that the blocks in the IDs array have with the
                      current block. The relationship can be ``VALUE`` or ``CHILD`` . A relationship
                      of type VALUE is a list that contains the ID of the VALUE block that's
                      associated with the KEY of a key-value pair. A relationship of type CHILD is a
                      list of IDs that identify WORD blocks.

                    - **Ids** *(list) --*

                      An array of IDs for related blocks. You can get the type of the relationship
                      from the ``Type`` element.

                      - *(string) --*

                - **EntityTypes** *(list) --*

                  The type of entity. The following can be returned:

                  * *KEY* - An identifier for a field on the document.

                  * *VALUE* - The field text.

                   ``EntityTypes`` isn't returned by ``DetectDocumentText`` and
                   ``GetDocumentTextDetection`` .

                  - *(string) --*

                - **SelectionStatus** *(string) --*

                  The selection status of a selection element, such as an option button or check
                  box.

                - **Page** *(integer) --*

                  The page on which a block was detected. ``Page`` is returned by asynchronous
                  operations. Page values greater than 1 are only returned for multipage documents
                  that are in PDF format. A scanned image (JPEG/PNG), even if it contains multiple
                  document pages, is considered to be a single-page document. The value of ``Page``
                  is always 1. Synchronous operations don't return ``Page`` because every input
                  document is considered to be a single-page document.

            - **Warnings** *(list) --*

              A list of warnings that occurred during the text-detection operation for the document.

              - *(dict) --*

                A warning about an issue that occurred during asynchronous text analysis (
                StartDocumentAnalysis ) or asynchronous document text detection (
                StartDocumentTextDetection ).

                - **ErrorCode** *(string) --*

                  The error code for the warning.

                - **Pages** *(list) --*

                  A list of the pages that the warning applies to.

                  - *(integer) --*

            - **StatusMessage** *(string) --*

              The current status of an asynchronous text-detection operation for the document.

            - **DetectDocumentTextModelVersion** *(string) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def start_document_analysis(
        self,
        DocumentLocation: ClientStartDocumentAnalysisDocumentLocationTypeDef,
        FeatureTypes: List[Literal["TABLES", "FORMS"]],
        ClientRequestToken: str = None,
        JobTag: str = None,
        NotificationChannel: ClientStartDocumentAnalysisNotificationChannelTypeDef = None,
    ) -> ClientStartDocumentAnalysisResponseTypeDef:
        """
        Starts the asynchronous analysis of an input document for relationships between detected
        items such as key-value pairs, tables, and selection elements.

         ``StartDocumentAnalysis`` can analyze text in documents that are in JPEG, PNG, and PDF
         format. The documents are stored in an Amazon S3 bucket. Use  DocumentLocation to specify
         the bucket name and file name of the document.

         ``StartDocumentAnalysis`` returns a job identifier (``JobId`` ) that you use to get the
         results of the operation. When text analysis is finished, Amazon Textract publishes a
         completion status to the Amazon Simple Notification Service (Amazon SNS) topic that you
         specify in ``NotificationChannel`` . To get the results of the text analysis operation,
         first check that the status value published to the Amazon SNS topic is ``SUCCEEDED`` . If
         so, call  GetDocumentAnalysis , and pass the job identifier (``JobId`` ) from the initial
         call to ``StartDocumentAnalysis`` .

        For more information, see `Document Text Analysis
        <https://docs.aws.amazon.com/textract/latest/dg/how-it-works-analyzing.html>`__ .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/textract-2018-06-27/StartDocumentAnalysis>`_

        **Request Syntax**
        ::

          response = client.start_document_analysis(
              DocumentLocation={
                  'S3Object': {
                      'Bucket': 'string',
                      'Name': 'string',
                      'Version': 'string'
                  }
              },
              FeatureTypes=[
                  'TABLES'|'FORMS',
              ],
              ClientRequestToken='string',
              JobTag='string',
              NotificationChannel={
                  'SNSTopicArn': 'string',
                  'RoleArn': 'string'
              }
          )
        :type DocumentLocation: dict
        :param DocumentLocation: **[REQUIRED]**

          The location of the document to be processed.

          - **S3Object** *(dict) --*

            The Amazon S3 bucket that contains the input document.

            - **Bucket** *(string) --*

              The name of the S3 bucket.

            - **Name** *(string) --*

              The file name of the input document. Synchronous operations can use image files that
              are in JPEG or PNG format. Asynchronous operations also support PDF format files.

            - **Version** *(string) --*

              If the bucket has versioning enabled, you can specify the object version.

        :type FeatureTypes: list
        :param FeatureTypes: **[REQUIRED]**

          A list of the types of analysis to perform. Add TABLES to the list to return information
          about the tables that are detected in the input document. Add FORMS to return detected
          form data. To perform both types of analysis, add TABLES and FORMS to ``FeatureTypes`` .
          All lines and words detected in the document are included in the response (including text
          that isn't related to the value of ``FeatureTypes`` ).

          - *(string) --*

        :type ClientRequestToken: string
        :param ClientRequestToken:

          The idempotent token that you use to identify the start request. If you use the same token
          with multiple ``StartDocumentAnalysis`` requests, the same ``JobId`` is returned. Use
          ``ClientRequestToken`` to prevent the same job from being accidentally started more than
          once. For more information, see `Calling Amazon Textract Asynchronous Operations
          <https://docs.aws.amazon.com/textract/latest/dg/api-async.html>`__ .

        :type JobTag: string
        :param JobTag:

          An identifier that you specify that's included in the completion notification published to
          the Amazon SNS topic. For example, you can use ``JobTag`` to identify the type of document
          that the completion notification corresponds to (such as a tax form or a receipt).

        :type NotificationChannel: dict
        :param NotificationChannel:

          The Amazon SNS topic ARN that you want Amazon Textract to publish the completion status of
          the operation to.

          - **SNSTopicArn** *(string) --* **[REQUIRED]**

            The Amazon SNS topic that Amazon Textract posts the completion status to.

          - **RoleArn** *(string) --* **[REQUIRED]**

            The Amazon Resource Name (ARN) of an IAM role that gives Amazon Textract publishing
            permissions to the Amazon SNS topic.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'JobId': 'string'
            }
          **Response Structure**

          - *(dict) --*

            - **JobId** *(string) --*

              The identifier for the document text detection job. Use ``JobId`` to identify the job
              in a subsequent call to ``GetDocumentAnalysis`` . A ``JobId`` value is only valid for
              7 days.
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def start_document_text_detection(
        self,
        DocumentLocation: ClientStartDocumentTextDetectionDocumentLocationTypeDef,
        ClientRequestToken: str = None,
        JobTag: str = None,
        NotificationChannel: ClientStartDocumentTextDetectionNotificationChannelTypeDef = None,
    ) -> ClientStartDocumentTextDetectionResponseTypeDef:
        """
        Starts the asynchronous detection of text in a document. Amazon Textract can detect lines of
        text and the words that make up a line of text.

         ``StartDocumentTextDetection`` can analyze text in documents that are in JPEG, PNG, and PDF
         format. The documents are stored in an Amazon S3 bucket. Use  DocumentLocation to specify
         the bucket name and file name of the document.

         ``StartTextDetection`` returns a job identifier (``JobId`` ) that you use to get the
         results of the operation. When text detection is finished, Amazon Textract publishes a
         completion status to the Amazon Simple Notification Service (Amazon SNS) topic that you
         specify in ``NotificationChannel`` . To get the results of the text detection operation,
         first check that the status value published to the Amazon SNS topic is ``SUCCEEDED`` . If
         so, call  GetDocumentTextDetection , and pass the job identifier (``JobId`` ) from the
         initial call to ``StartDocumentTextDetection`` .

        For more information, see `Document Text Detection
        <https://docs.aws.amazon.com/textract/latest/dg/how-it-works-detecting.html>`__ .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/textract-2018-06-27/StartDocumentTextDetection>`_

        **Request Syntax**
        ::

          response = client.start_document_text_detection(
              DocumentLocation={
                  'S3Object': {
                      'Bucket': 'string',
                      'Name': 'string',
                      'Version': 'string'
                  }
              },
              ClientRequestToken='string',
              JobTag='string',
              NotificationChannel={
                  'SNSTopicArn': 'string',
                  'RoleArn': 'string'
              }
          )
        :type DocumentLocation: dict
        :param DocumentLocation: **[REQUIRED]**

          The location of the document to be processed.

          - **S3Object** *(dict) --*

            The Amazon S3 bucket that contains the input document.

            - **Bucket** *(string) --*

              The name of the S3 bucket.

            - **Name** *(string) --*

              The file name of the input document. Synchronous operations can use image files that
              are in JPEG or PNG format. Asynchronous operations also support PDF format files.

            - **Version** *(string) --*

              If the bucket has versioning enabled, you can specify the object version.

        :type ClientRequestToken: string
        :param ClientRequestToken:

          The idempotent token that's used to identify the start request. If you use the same token
          with multiple ``StartDocumentTextDetection`` requests, the same ``JobId`` is returned. Use
          ``ClientRequestToken`` to prevent the same job from being accidentally started more than
          once. For more information, see `Calling Amazon Textract Asynchronous Operations
          <https://docs.aws.amazon.com/textract/latest/dg/api-async.html>`__ .

        :type JobTag: string
        :param JobTag:

          An identifier that you specify that's included in the completion notification published to
          the Amazon SNS topic. For example, you can use ``JobTag`` to identify the type of document
          that the completion notification corresponds to (such as a tax form or a receipt).

        :type NotificationChannel: dict
        :param NotificationChannel:

          The Amazon SNS topic ARN that you want Amazon Textract to publish the completion status of
          the operation to.

          - **SNSTopicArn** *(string) --* **[REQUIRED]**

            The Amazon SNS topic that Amazon Textract posts the completion status to.

          - **RoleArn** *(string) --* **[REQUIRED]**

            The Amazon Resource Name (ARN) of an IAM role that gives Amazon Textract publishing
            permissions to the Amazon SNS topic.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'JobId': 'string'
            }
          **Response Structure**

          - *(dict) --*

            - **JobId** *(string) --*

              The identifier of the text detection job for the document. Use ``JobId`` to identify
              the job in a subsequent call to ``GetDocumentTextDetection`` . A ``JobId`` value is
              only valid for 7 days.
        """


class Exceptions:
    AccessDeniedException: Boto3ClientError
    BadDocumentException: Boto3ClientError
    ClientError: Boto3ClientError
    DocumentTooLargeException: Boto3ClientError
    HumanLoopQuotaExceededException: Boto3ClientError
    IdempotentParameterMismatchException: Boto3ClientError
    InternalServerError: Boto3ClientError
    InvalidJobIdException: Boto3ClientError
    InvalidParameterException: Boto3ClientError
    InvalidS3ObjectException: Boto3ClientError
    LimitExceededException: Boto3ClientError
    ProvisionedThroughputExceededException: Boto3ClientError
    ThrottlingException: Boto3ClientError
    UnsupportedDocumentException: Boto3ClientError
